#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <pthread.h>

#include "lvgl/lvgl.h"
#include "lv_examples/lv_examples.h"

#ifdef SIMULATOR
#include <SDL2/SDL.h>

#include "lv_drivers/display/monitor.h"
#include "lv_drivers/indev/mouse.h"
#else
#include "lv_drivers/display/fbdev.h"
#include "lv_drivers/indev/evdev.h"
#endif

void tick_thread(void)
{
    while(1) {
        usleep(5000);
        lv_tick_inc(5);
    }
}

void data_thread(void)
{
    // TODO: get data from i2c and other device.
}

void fbusb_set_screen_mirror(char mode)
{
    FILE *fp = fopen("/sys/devices/platform/101c0000.ehci/usb1/1-1/1-1:1.0/command", "wb");
    if (fp == NULL)
        return;

    char buf[] = {0x00, 0x36, 0x02, 0x00, 0x00, 0x00, 0x00, 0x00};
    buf[6] = mode;
    fwrite(buf, 1, sizeof(buf), fp);
    fclose(fp);
}

void init(void)
{
    // mirror the screen x,y to fit the framebuffer data.
    fbusb_set_screen_mirror(2);
    
    // disable the fbcon and cursor.
    system("echo 0 > /sys/devices/virtual/vtconsole/vtcon1/bind");
    system("echo 0 > /sys/devices/virtual/graphics/fbcon/cursor_blink");
    
    lv_init();

    static lv_disp_buf_t disp_buf;
    static lv_color_t buffer[480 * 64];
    lv_disp_buf_init(&disp_buf, buffer, NULL, 480 * 64);

    lv_disp_drv_t disp_drv;
    lv_disp_drv_init(&disp_drv);
    disp_drv.buffer = &disp_buf;
#ifdef SIMULATOR
    monitor_init();
    disp_drv.flush_cb = monitor_flush;
#else
    fbdev_init();
    disp_drv.flush_cb = fbdev_flush;
#endif
    lv_disp_drv_register(&disp_drv);

#ifdef SIMULATOR
    mouse_init();
    lv_indev_drv_t indev_drv;
    lv_indev_drv_init(&indev_drv);
    indev_drv.type = LV_INDEV_TYPE_POINTER;
    indev_drv.read_cb = mouse_read;
    lv_indev_t * mouse_indev = lv_indev_drv_register(&indev_drv);

    LV_IMG_DECLARE(mouse_cursor_icon);
    lv_obj_t * cursor_obj =  lv_img_create(lv_disp_get_scr_act(NULL), NULL);
    lv_img_set_src(cursor_obj, &mouse_cursor_icon);
    lv_indev_set_cursor(mouse_indev, cursor_obj);
#else
    evdev_init();
    lv_indev_drv_t indev_drv;
    lv_indev_drv_init(&indev_drv);
    indev_drv.type = LV_INDEV_TYPE_POINTER;
    indev_drv.read_cb = evdev_read;
    lv_indev_drv_register(&indev_drv);
#endif
    
    pthread_t pid;
    pthread_create(&pid, NULL, tick_thread, NULL);
    pthread_create(&pid, NULL, data_thread, NULL);
}

int main(int argc, char *argv[])
{
    int counter = 0;
    
    init();
    lv_demo_widgets();
   
    while(1) {
        lv_task_handler();   
        usleep(5000);   // trigger every 5ms.
        counter++;

#ifdef SIMULATOR
        SDL_Event event;
        while(SDL_PollEvent(&event)) {
            mouse_handler(&event);
        }
#endif
    }

    return 0;
}
